#!/usr/bin/env python

from __future__ import division

import os
import getpass

import delivery
import report
import transfer
import helper
import subproc
import error

from deliverypathconfig import WORKINGDIRROOT, DELIVERYDIR

def fileload(hostname, hostpath, filename, log, 
             targetpath="~/.", 
             hostport=22):
    """
    
    Purpose
    =======
    Move reference files to be delivered from a local machine to the production machine.
    At this time the production machine is knows as smalls.
    
    """
    # == Step 1 get data from remote machine ==
    
    # Get username for remote system
    default_username = getpass.getuser()
    username = raw_input('Username [%s]: ' % default_username)
    if len(username) == 0:
        username = default_username

    # Get password for remote system
    password = getpass.getpass('Password for %s@%s: ' % (username, hostname))

    
    transfer.transfer_files(username, password, hostname, hostpath, targetpath,
                            filename, hostport=hostport, log=log)
    
def fileload_web(username, password, hostname, hostpath, targetpath,
                 filename, hostport=22, log=None):
    """
    
    Purpose
    =======
    Move reference files to be delivered from a local machine to the production machine.
    This function serves the same purpose as fileload() except that it does not prompt the user.
    Instead it assumes the appropriate input is coming from the GUI interface.
    
    """
    
    transfer.transfer_files(username, password, hostname, hostpath, targetpath,
                            filename, hostport, log)


def certify(filename, comments='Y', change_level="", opus_flag='Y', pedigree="", log=None):
    """
    
    Purpose
    =======
    Make certain that reference files conform to FITS standard and match CDBS template files.
    
    """
    helper.updatelog(log,"\n")
    helper.updatelog(log,"\n\n***** BEGIN CERTIFY *****\n\n")
    delivery.set_file_permissions(filename, log)
    delivery.check_headers(filename, log)
    delivery.check_FITS_standard(filename, log)
    delivery.CDBS_certify(filename, log)
    delivery.create_load_file(filename, log)
    lodfilename = helper.getlodfilename(filename)
    delivery.populate_load_file(filename,lodfilename, comments, change_level, opus_flag,
                                pedigree, log)
    (fullpathfilename, path, filename) = helper.fileinfo(lodfilename)
    os.remove(lodfilename)
    os.rename(path+"/pop_"+filename,lodfilename)
    helper.updatelog(log,"\n\n***** END CERTIFY *****\n\n")
    return None

def certifylod(filename,log=None):
    """
    
    Purpose
    =======
    Given the name of a FITS reference file deduce the name of the related
    LOD file and take it through the certify process.
    
    args:
    
    :filename:  NAME of FITS file being delivered to CDBS
    
    """
    # This line of code is assuming that the extension of the fits file
    lodfilename = filename[:-len(".fits")] + ".lod"
    helper.updatelog(log,"\n\n***** BEGIN CERTIFYLOD *****\n\n")
    delivery.certify_load_file(lodfilename, log)

    # Only expload image FITS files 
    if helper.getfiletype(filename) == "image":
        # Only reference files for certain instruments use wildcards
        instrument = helper.getinstname(filename)
        if instrument in ['acs', 'wfc3', 'stis']:
            delivery.expload_load_file(lodfilename, instrument, log)

    delivery.run_check_load(lodfilename, log)
    newfilename = delivery.rename_ref_files(filename, log)
    helper.updatelog(log,filename+" -> "+newfilename+"\n\n")
    helper.updatelog(log,"\n\n***** END CERTIFYLOD *****\n\n")
    return newfilename
    
def deliver(filelist, log=None):
    """

    Purpose
    =======
    Create the LOD files and execute the programs that actually update the CDBS database with
    the new reference files.  The program assumes that all files in the delivery will be
    moved to the same delivery area on CDBS.
    
    Arguments
    =========
    :filelist: List of FITS files to be delivered to CDBS
    
    """
    
    # Copy all files to the CDBS files working directory
    workingdir = transfer.create_dir_target(filelist[0],root=WORKINGDIRROOT)
    if helper.getcurdir() != workingdir:
        transfer.mkdir_nested(workingdir)
        (code, out, err) = subproc.execute(["cp", '*', workingdir+"/."], log=log)
        if code != 0:
            helper.updatelog(log,err)
            raise error.CDBSError(out, err, code)

    # Copy FITS and LOD files to CDBS delivery staging area
    # Step 1: Define target directory
    directory = transfer.create_dir_target(filelist[0],root=DELIVERYDIR)
    
    # Create target directory
    transfer.mkdir_nested(directory)

    # Copy Files to delivery area 
    delivery.dms_delivery(directory, workingdir, log)
    
    #delivery.deliver_delivery_form(filename, log)
    
def reporting(filename, log=None):
    """
    
    Purpose
    =======
    Create reports associated with a new reference file delivery
    
    """
    report.run_cdbs_report(filename, log)
    report.check_file_sizes(filename, log)
    report.check_file_installation(filename, log)
    report.send_notifications(filename, log)
    report.update_wiki(filename, log)

